/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  DRV84xx.h - Declaration file for utility functions and global variables
 *  DRV84xxEVM_FIRMWARE
 *  10/24/2020
 *
 ******************************************************************************/
 #ifndef drv8436_H_
 #define drv8436_H_

/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "msp430.h"
#include "dataTypeDefinition.h"
/******************************************************************************
 * MACROS
 *****************************************************************************/

/*
 * GPIO Port 1 Definitions
 */
#define nSLEEP             BIT0         /**<P1.0: Sleep pin*/

/*
 * GPIO Port 2 Definitions
 */
#define M1_PD              BIT0         // P2.0: M1 330k pulldown
#define STL_MODE           BIT1         // P2.1: DECAY1 for STEP/DIR variant
//#define ADECAY             BIT1         // P2.1: ADECAY for PH/EN and PWM variants
#define STL_MODE_PD        BIT2         // P2.2: ADECAY 330k pulldown for PH/EN and PWM variants
//#define DECAY0             BIT3         // P2.3: DECAY0 for for STEP/DIR variant
//#define BDECAY             BIT3         // P2.3: BDECAY for PH/EN and PWM variants
#define TRQ_PD             BIT4         // P2.4: BDECAY 330k pulldown for PH/EN and PWM variants
#define STL_REP            BIT5         // P2.5: TOFF pin
#define STL_REP_PD         BIT6         // P2.6: TOFF 330k pulldown
#define nFAULT             BIT7         // P2.7: Fault indication pin (with port interrupt)

/*
 * GPIO Port 3 Definitions
 */
#define RX                 BIT4         // P3.4
#define TX                 BIT5         // P3.5

/*
 * GPIO Port 4 Definitions
 */
#define M1                 BIT1         // P4.1: M1 Pin for STEP/DIR variant
#define BPH                BIT1         // P4.1: BPH pin for PH/EN variant
#define BIN2               BIT1         // P4.1: BIN2 pin for PWM variant
#define STEP               BIT2         // P4.2: STEP Pin for STEP/DIR variant
#define BEN                BIT2         // P4.2: BEN pin for PH/EN variant
#define BIN1               BIT2         // P4.2: BIN1 pin for PWM variant
#define DIR                BIT3         // P4.3: DIR Pin for STEP/DIR variant
#define APH                BIT3         // P4.3: APH pin for PH/EN variant
#define AIN2               BIT3         // P4.3: AIN2 pin for PWM variant
#define ENABLE             BIT4         // P4.4: ENABLE Pin for STEP/DIR variant
#define AEN                BIT4         // P4.4: AEN pin for PH/EN variant
#define AIN1               BIT4         // P4.4: AIN1 pin for PWM variant

/*
 * GPIO Port 6 Definitions
 */
//#define CHANGE_M0_AND_TRQ_CNT         // this can be used to redefine the location of M0 and TRQ_CNT

#define RDSON0             BIT0         // P6.0 RDSON variant BIT0
#define RDSON1             BIT1         // P6.1 RDSON variant BIT1
#define RDSON2             BIT2         // P6.2 RDSON variant BIT2
#define INTERFACE          BIT3         // P6.3 Interface variant bit
#define STATUS             BIT4         // P6.4 STATUS LED bit

// this is based on using DAC0 at P6.6 and DAC1 at P6.7
#ifdef CHANGE_M0_AND_TRQ_CNT
#define TRQ_CNT_STL_TH     BIT5         // P6.5 TRQ_CNT/STL_TH ADC or DAC
#define VREF               BIT6         // P6.6 DAC for VREF for STEP/DIR variant
#define M0                 BIT7         // P6.7 M0 pin for STEP/DIR variant
#else
#define M0                 BIT5         // P6.7 M0 pin for STEP/DIR variant
#define VREF               BIT6         // P6.6 DAC for VREF for STEP/DIR variant
#define TRQ_CNT_STL_TH     BIT7         // P6.5 TRQ_CNT/STL_TH ADC or DAC
#endif

#define INTERNAL_THRESHOLD      0       // use internal threshold mode on STL_MODE
#define LEARN                   1       // use learn mode on STL_MODE
#define EXTERNAL_THRESHOLD      2       // use external threshold mode on STL_MODE

#define Disable                 0
#define Enable                  1

/*Firmware version*/
#define FW_VERSION   0.44               // added new GUI features in v0.1.6

#ifdef SET_DECAY_MODE
/* Decay mode */
#define STDD_STDD               0       /* Smart Tune Dynamic Control   */
#define STRC_STRC               1       /* Smart Tune Ripple Control */
#define MIX30_MIX30             2       /* Mixed Decay 30% fast */
#define SLOW_MIX30              3       /* Increasing steps Slow; Decreasing steps Mixed decay30% fast */
#define MIX60_MIX60             4       /* Mixed Decay 60% fast */
#define SLOW_SLOW               5       /* Slow Decay */

/* TOFF */
#define TOFF_7u                 0       /* 7uS TOFF time */
#define TOFF_16u                1       /* 16uS TOFF time */
#define TOFF_24u                2       /* 24uS TOFF time */
#define TOFF_32u                3       /* 32uS TOFF time */

// Set TOFF
#define set_TOFF_7us              P2DIR &= ~(TOFF + TOFF_PD)
#define set_TOFF_16us             P2DIR |= TOFF; P2DIR &= ~TOFF_PD; P2OUT &= ~TOFF
#define set_TOFF_24us             P2DIR |= TOFF; P2DIR &= ~TOFF_PD; P2OUT |= TOFF
#define set_TOFF_32us             P2DIR &= ~TOFF; P2DIR |= TOFF_PD; P2OUT &= ~TOFF_PD

// Set decay modes for STEP/DIR variant
#define set_dynamic_decay        P2DIR |= DECAY0 + DECAY1; P2DIR &= ~(ADECAY_PD + BDECAY_PD); P2OUT &= ~(DECAY0 + DECAY1)
#define set_ripple_control       P2DIR |= DECAY0 + DECAY1; P2DIR &= ~(ADECAY_PD + BDECAY_PD); P2OUT &= ~DECAY1; P2OUT |= DECAY0
#define set_mixed30_mixed30      P2DIR |= DECAY0 + DECAY1; P2DIR &= ~(ADECAY_PD + BDECAY_PD); P2OUT |= DECAY1; P2OUT &= ~DECAY0
#define set_slow_mixed30         P2DIR |= DECAY0 + DECAY1; P2DIR &= ~(ADECAY_PD + BDECAY_PD); P2OUT |= DECAY0 + DECAY1
#define set_mixed60_mixed60      P2DIR |= DECAY0; P2DIR &= ~(DECAY1 + ADECAY_PD + BDECAY_PD); P2OUT &= ~DECAY0
#define set_slow_slow            P2DIR |= DECAY0; P2DIR &= ~(DECAY1 + ADECAY_PD + BDECAY_PD); P2OUT |= DECAY0
#endif

/* microstepping */
#define FULLSTEP_100            0       /* Full step (2-phase excitation) with 100% current */
#define FULLSTEP_71             1       /* Full step (2-phase excitation) with 71% current */
#define NONCIRCULAR_HALFSTEP    2       /* Non-circular 1/2 step */
#define HALFSTEP                3       /* 1/2 step */
#define MICROSTEP_4             4       /* 1/4 step*/
#define MICROSTEP_8             5       /* 1/8 step*/
#define MICROSTEP_16            6       /* 1/16 step*/
#define MICROSTEP_32            7       /* 1/32 step*/
#define MICROSTEP_64            8       /* 1/64 step*/
#define MICROSTEP_128           9       /* 1/128 step*/
#define MICROSTEP_256           10      /* 1/256 step*/

/* Enable nSLEEP*/
#define set_nSLEEP_hi            P1OUT |= nSLEEP
#define set_nSLEEP_lo            P1OUT &= ~nSLEEP

/* ENABLE*/
#define set_ENABLE_hi            P4DIR |= ENABLE; P4OUT |= ENABLE
#define set_ENABLE_lo            P4DIR |= ENABLE; P4OUT &= ~ENABLE
#define set_ENABLE_zz            P4DIR &= ~ENABLE

/* Enable DIR*/
#define set_DIR_hi               P4OUT |= DIR
#define set_DIR_lo               P4OUT &= ~DIR

/* Enable STEP*/
#define set_STEP_hi               P4OUT |= STEP
#define set_STEP_lo               P4OUT &= ~STEP

// Set microstepping
#define set_M0_hi                 P6DIR |= M0; P6OUT |= M0
#define set_M0_lo                 P6DIR |= M0; P6OUT &= ~M0
#define set_M0_hiz                P6DIR &= ~M0
#define set_M1_hi                 P4DIR |= M1; P4OUT |= M1; P2DIR &= ~M1_PD
#define set_M1_lo                 P4DIR |= M1; P4OUT &= ~M1; P2DIR &= ~M1_PD
#define set_M1_hiz                P4DIR &= ~M1; P2DIR &= ~M1_PD
#define set_M1_330k               P4DIR &= ~M1; P2DIR |= M1_PD; P2OUT &= ~M1_PD

#define set_fullstep_100          set_M0_lo; set_M1_lo
#define set_fullstep_71           set_M0_lo; set_M1_330k
#define set_noncircular_halfstep  set_M0_hi; set_M1_lo
#define set_halfstep              set_M0_hiz; set_M1_lo
#define set_microstep_4           set_M0_lo; set_M1_hi
#define set_microstep_8           set_M0_hi; set_M1_hi
#define set_microstep_16          set_M0_hiz; set_M1_hi
#define set_microstep_32          set_M0_lo; set_M1_hiz
#define set_microstep_64          set_M0_hiz; set_M1_330k
#define set_microstep_128         set_M0_hiz; set_M1_hiz
#define set_microstep_256         set_M0_hi; set_M1_hiz

// Set StallMode
#define set_StallMode_lo          P2DIR |= STL_MODE; P2OUT &= ~STL_MODE
#define set_StallMode_hi          P2DIR |= STL_MODE; P2OUT |= STL_MODE
#define set_StallMode_hiz         P2DIR &= ~STL_MODE

// Set TRQCNTvalue
#define set_TRQCNT_value          DAC12_1CTL |= DAC12AMP_7; DAC12_1DAT =
#define set_TRQCNT_to_input       DAC12_1CTL &= ~DAC12AMP_7

// Read nFAULT
#define read_nFAULT_pin          (P2IN & nFAULT)

// Read STL_REP
#define read_StlRep_pin          (P2IN & STL_REP)

// Disable Stall Reporting by externally forcing STL_REP low
#define disable_StlRep            P2DIR |= STL_REP; P2REN &= ~STL_REP; P2OUT &= ~STL_REP

// Enable Stall Reporting by no external low and mcu pin in input mode with pull
#define enable_StlRep             P2DIR &= ~STL_REP; P2REN |= STL_REP; P2OUT |= STL_REP

// Toggle status LED
#define toggle_status_led         P6OUT ^=  STATUS

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/

typedef struct DRV843x_Device_Obj
{

  uint16_t DeviceID;                // DRV84xx - Read IC_ID register and set the value
  uint16_t RDSON;                   // DRV84xx - RDSON variant variable
  uint16_t VM_variant;              // DRV84xx - VM variant

  float Firmware_Version;

  /* Device Parameters*/
  volatile uint8_t nSleep;          // 0: Device Sleep , 1: Device Awake
  volatile uint8_t gEnable;         // 0: Outputs high Hi-Z, 1: Outputs active
  volatile uint8_t OCP_latched;     // 0: automatic retry, 1: latched shutdown
  volatile uint8_t nFault;          // 0: No fault , 1: Fault
  volatile uint8_t ClrFlt;
  volatile uint8_t StallMode;       // variable for STL_MODE setting
  uint8_t StallRep;                 // variable enable and disable stall reporting
  uint8_t    MicroSTEP;             // variable for microstep modes setting

  /* Register Read Write */
   uint16_t   ManWriteAddr;
   uint16_t   ManReadAddr;
   uint16_t   ManWriteData;
   uint16_t   ManReadData;
   bool       ManWriteCmd;
   bool       ManReadCmd;
   bool       ReadCmd;

//   volatile uint8_t StallMode;
   volatile uint8_t STLREP;

//   Dedicated Torque Count global variables for each of the three modes
   float TRQCNTvalue0;
   float TRQCNTvalue1;
   float TRQCNTvalue2;


} DRV843x_Device_Obj_t;

extern volatile DRV843x_Device_Obj_t gDRV843x_Obj;

/*****************************************************************************
EVM variants
*****************************************************************************/

/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
//extern volatile TRGT_INFO_Obj targetInfo;

/*****************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
 * API Prototypes
 *****************************************************************************/
void mcu_init();
/**
* @brief     :This function, initializes all the peripherals and DRV843x related GPIOs and registers\n
 * @param[in] void There is no argument
 * @return    void (none)
 */
void drv843x_init(void);
/**
 * @brief     :Initialize DRV834x IC
 * @param[in] : Void
 * @return    : No Return
 */

/*****************************************************************************/

/* Support for C++ codebase */
#ifdef __cplusplus
}
#endif

#endif /* drv89xxq1_H_ */
