/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  stepper_ISR.h - Declaration file for functions and global variables related to stepper control
 *  DRV843x_EVM_FIRMWARE
 *  11/29/2018
 *
 ******************************************************************************/

#include <drv84xx/include/drv84xx.h>
#include <msp430.h>
#include "stepper.h"
#include <intrinsics.h>

#pragma vector= DAC12_VECTOR, DMA_VECTOR, USCIAB1TX_VECTOR, USCIAB1RX_VECTOR, \
                PORT1_VECTOR, PORT2_VECTOR,ADC12_VECTOR, COMPARATORA_VECTOR, \
                TIMERA0_VECTOR, TIMERB0_VECTOR, NMI_VECTOR
__interrupt void Trap_ISR(void)
{
}

#pragma vector=TIMERA1_VECTOR
__interrupt void TimerA1(void)
{
    switch (TAIV)
    {
    case TAIV_TACCR1:
        // increment or decrement to the new timer A value
       if (gStepper_Obj.AccelerateState == ACCEL && (SteppingRate <= DesiredTargetSpeed))
       {
           // increase pps and calculate new pps
           if (SteppingRate + SteppingRateUpdate >= DesiredTargetSpeed)
           {
               gStepper_Obj.AccelerateState = RUN;                              // No longer accelerate
               SteppingRate = DesiredTargetSpeed;
            }
           else
           {
               SteppingRate += SteppingRateUpdate;                 // Increment
               gStepper_Obj.AccelerateState = ACCEL;
           }
       }
       if (gStepper_Obj.AccelerateState == DECEL)
       {
           if (SteppingRate >= StoppingSpeed)
           {
               if (SteppingRate <= StoppingSpeed + SteppingRateUpdate)
               {
                   if (MoveSteps == false && SteppingRate == StoppingSpeed)
                   gStepper_Obj.AccelerateState = IDLE;                             // Stop motor when speed is reached
                   SteppingRate = StoppingSpeed;
               }
               else
               {
                   SteppingRate -= SteppingRateUpdate;                 // Decrement
                   gStepper_Obj.AccelerateState = DECEL;
               }
           }
           else // already less than stopping speed
           {
               if (MoveSteps == false)
                   gStepper_Obj.AccelerateState = IDLE;                             // Stop motor when speed is reached
           }
       }
       // this allows for the motor to spin down to new speed without stopping
       if (gStepper_Obj.AccelerateState == DECEL2NEWTARGET)
       {
           if (SteppingRate >= DesiredTargetSpeed)
           {
           // decrease pps and calculate new pps
               if (SteppingRate - SteppingRateUpdate <= DesiredTargetSpeed)
               {
                   if (SteppingRate == DesiredTargetSpeed)
                   {
                       gStepper_Obj.AccelerateState = RUN;                              // switch to RUN when speed is reached
                   }
                   SteppingRate = DesiredTargetSpeed;
               }
               else
               {
                   SteppingRate -= SteppingRateUpdate;                 // Decrement
                   gStepper_Obj.AccelerateState = DECEL2NEWTARGET;
               }
           }
       }
       // do not perform the division here -- only update the stepping rate
       // if the stepping rate has been updated, set a flag
       if (SteppingRate != PriorSteppingRate)
       {
           UpdateSteppingRateTMR = true;                           // Set flag to re-calculate stepping rate TMR
           PriorSteppingRate = SteppingRate;
       }
       break;
    case TAIV_TACCR2:
        _nop();
        break;
    case TAIV_TAIFG:
        break;
    }

}


#pragma vector=TIMERB1_VECTOR
__interrupt void TimerB1(void)
{
    switch (TBIV)
    {
    case TBIV_TBCCR1:
       break;
    case TBIV_TBCCR2:
        if (MoveSteps)                              // if manual advance count steps
         {                                           // when complete
             tmpStepsToMove++;                      // increment step counter
             if (tmpStepsToMove >= Steps2Decel)
             {
                 gStepper_Obj.AccelerateState = DECEL;            // start deceleration
             }
             if (tmpStepsToMove >= StepsToMove)      // stop here when equal
             {
                 gStepper_Obj.AccelerateState = IDLE;
                 temp_reciprocation = false;
                 if((gStepper_Obj.RECIPROCATION == true && gStepper_Obj.MANUAL_ADVANCE == true))
                 {
                     tmpStepsToMove = 0;                         // clear counter
                     if(gStepper_Obj.gDIR == 1)                // change direction when finished the steps
                     {
                         gStepper_Obj.gDIR = 0;
                     }
                     else gStepper_Obj.gDIR = 1;
                 }
                 if(gStepper_Obj.RECIPROCATION == false)
                 {
                     if(last_reciprocation_state == 1)
                     {
                         if(gStepper_Obj.gDIR == 1)                   // reverse the direction when the motor is stopped
                         {
                             gStepper_Obj.gDIR = 0;
                         }
                         else gStepper_Obj.gDIR = 1;
                         last_reciprocation_state = 0;
                     }
                     MoveSteps = false;
                 }
                 TACTL &= ~MC_3;                     // Stop timerA
                 TBCTL &= ~MC_3;                     // Stop timerB
             }
         }
         //      Reload the timers below
         TBCCR0 = SteppingRateTMR;                   //
         TBCCR2 = SteppingRateTMR >> 1;              // set at 50%
        break;
    case TBIV_TBCCR3:
        break;
    case TBIV_TBCCR4:
        break;
    case TBIV_TBCCR5:
        break;
    case TBIV_TBCCR6:
        break;
    case TBIV_TBIFG:
        break;
    }
}

